const fs = require('fs');
const path = require('path');
const marked = require('marked');
const tempos = require('tempos/src');
const yaml = require('js-yaml');
const gulp = require('gulp');
const sass = require('gulp-ruby-sass');
const autoprefixer = require('autoprefixer');
const nano = require('gulp-cssnano');
const postcss = require('gulp-postcss');
const uglify = require('uglify-js');
const imagemin = require('gulp-imagemin');
const rename = require('gulp-rename');
const concat = require('gulp-concat');
const base64 = require('gulp-base64');
const rollup = require('rollup');
const json = require('rollup-plugin-json');
const babel = require('rollup-plugin-babel');
const resolve = require('rollup-plugin-node-resolve');
const replace = require('rollup-plugin-replace');
const commonjs = require('rollup-plugin-commonjs');

const RYAML = /^----([\s\S]+?)----([\s\S]+)/;
const base = __dirname;
const option = {};

// get directory
const getDir = (paths) => {
    let res;
    paths = path.resolve(base, paths);
    try{
        res = fs.readdirSync(paths);
    } catch(err) {
        throw `Directory "${paths}"  does not exist.`;
    }
    return res;
};


// create dir
const createDir = (paths) => {
    if( typeof paths !== 'string' ) {
        throw 'createDir: The parameter "paths" must be a string.';
    }
    const pa = paths.split(/\\/g);
    const len = pa.length;
    let index = 0;
    while( ++index <= len ) {
        let curp = pa.slice(0, index).join('\\');
        if( !fs.existsSync(curp) ) {
            fs.mkdirSync(curp);
        }
    }
}


// load yaml file
const yamlLoad = (paths, pattern) => {
    let res = {}, mat;
    let  file = fs.readFileSync( path.resolve(base, paths) ).toString();
    if( pattern ) {
        if( mat = file.match(pattern) ) {
            res.yaml = yaml.safeLoad(mat[1]);
            res.template = mat[2].trim();
        }
    }
    else {
        res.yaml = yaml.load(file);
    }
    return res;
};


// get files
// return all files while all is true
const getFiles = (paths, suffix = '.temp', all = false) => {
    const dir = getDir(paths);
    const res = {};

    dir.map( (f) => {
        const curPath = path.resolve(base, paths + '/' + f);
        const isDir = fs.statSync(curPath).isDirectory();
        if( !isDir ) {
            const r = f.split(/\./);
            const tmp = {};

            tmp.extname = ( '.' + r.pop() ).trim();
            tmp.filename = r.join('');
            tmp.filepath = curPath;
            
            if( !suffix || suffix === tmp.extname ) {
                let mat = yamlLoad(curPath, RYAML);
                Object.assign(tmp, mat.yaml);
                tmp.template = mat.template || '';

                if( '.md' === suffix ) {
                    tmp.template = marked(tmp.template);
                }

                res[ tmp.filename ] = tmp;
            }
        }
        else if(all) {
            res[ f ] = getFiles(curPath, suffix, all);
        }
    });

    return res;
};


// generated file
const writeFile = (res, compile, paths) => {
    fs.writeFileSync(paths, compile(res), 'utf8');
};

//
option.config = yamlLoad('./_config.yml').yaml;
option.proName = path.parse(path.resolve('.')).name;

option.theme_dir = './theme/' + option.config.theme;
option.source_dir = option.theme_dir + '/source/';
option.temp_dir = option.theme_dir + '/template/';
option.dest_dir = './' + option.config.generate_dir;

option.layout = getFiles(option.temp_dir);
option.post = getFiles('post', '.md', true);


// html
// gulp.task('html', () => {
    for( let key in option.layout ) {
        let it = option.layout[key];
        let postHook;
        it.compile = tempos.compile(null, it);
        it.post = option.post;
        const paths = path.resolve(base, option.dest_dir + '/' + it.filename);
        if( postHook = it.post[ it.filename ] ) {
            createDir(paths);
            for( let k in postHook ) {
                writeFile(postHook[k], it.compile, path.resolve(paths, option.filename + '.html'));
            }
        }
        else {
            writeFile(it, it.compile, paths + '.html');
        }
    }
// });


// copy
gulp.task('copy', () => {
    gulp.src(option.source_dir+'image/**/*')
        .pipe(imagemin({
            optimizationLevel: 7,
            progressive: true,
            interlaced: true
        }))
        .pipe(gulp.dest(option.dest_dir + '/image'));
        
    gulp.src(option.source_dir+'plugin/**/*')
        .pipe(gulp.dest(option.dest_dir));
});


// styles
gulp.task('styles', () => {
	return sass(option.source_dir + '/styles/index.scss')
		.pipe(concat(option.proName + '.min.css'))
		.pipe(postcss([
			autoprefixer({
				browsers: [
					'last 2 versions',
					'last 2 Chrome versions',
					'> 5%',
					'> 5% in US',
					'ie >= 9',
					'Firefox >= 20',
					'Firefox <= 20',
					'iOS 7'
				]
			})
		]))
		.pipe(base64())
		.pipe(nano())
		.pipe(gulp.dest(option.dest_dir + '/styles'));
});


const rollup_cfg = {
    entry: option.source_dir + 'scripts/index.js',
    dest: option.dest_dir + '/scripts/'+ option.proName +'.js',
    format: 'umd',
    moduleName: option.proName,
    sourceMap: true,
    plugins: [
        json(),
        resolve({
            jsnext: true,
            main: true
        }),
        commonjs({
            ignoreGlobal: true
        }),
        babel({
            exclude: ['node_modules/**'],
            presets: ['es2015-rollup']
        }),
        replace({
            exclude: 'node_modules/**',
            ENV: JSON.stringify(process.env.NODE_ENV || 'development')
        })
    ]
};

gulp.task('scripts', () => {
    createDir(option.dest_dir+'/scripts/');
    rollup.rollup(rollup_cfg).then( (bundle) => {
        const code = bundle.generate(rollup_cfg).code;

        const res = process.env.NODE_ENV === 'production' ? uglify.minify(code, {
            output: {
                ascii_only: true
            },
            compress: {
                pure_funcs: ['makeMap']
            }
        }).code : code;

        fs.writeFileSync(rollup_cfg.dest, res, 'utf8');
    });
});


// Default task
gulp.task('default', () => {
	gulp.start(['copy', 'html', 'styles', 'scripts']);
});


// Watch
gulp.task('watch', () => {
    gulp.watch([
        './theme/'+option.config.theme+'/**/*',
        './post/**/*'
    ], ['html', 'styles', 'scripts']);
});